/******************************************************************************
 * GeoIP support functions. Allow to query the GEOIP table for the country
 * code or country name associated to an IP address. To make it more easy to
 * use this package supplies both functions for searching for a dotted IP address
 * or their converted ULONG value, depending on how you call the functions.
 * Written by Sascha 'SieGeL' Pfalz and licensed under the GPL
 ******************************************************************************/

/*
 *  Package Header:
 */

CREATE OR REPLACE PACKAGE UTL_GEOIP AS
  FUNCTION GETCC(IPADDR NUMBER) RETURN VARCHAR2;
  FUNCTION GETCC(IPADDR VARCHAR2) RETURN VARCHAR2;
  FUNCTION GETCOUNTRYNAME(IPADDR NUMBER) RETURN VARCHAR2;
  FUNCTION GETCOUNTRYNAME(IPADDR VARCHAR2) RETURN VARCHAR2;
END UTL_GEOIP;
/

/*
 *  Package body:
 */

CREATE OR REPLACE PACKAGE BODY UTL_GEOIP AS  

  /*
   *  IP2LONG() takes a dotted internet address in the format w.x.y.z 
   *  and returns the long value of it
   */

  FUNCTION IP2LONG(IPADDR VARCHAR2) RETURN NUMBER
  IS
    v_part1     NUMBER(38);
    v_part2     NUMBER(38);
    v_part3     NUMBER(38);
    v_part4     NUMBER(38);
    v_dummy     VARCHAR2(20);
  BEGIN
    v_dummy:=SUBSTR(IPADDR,1,INSTR(IPADDR,'.',1,1)-1);
    IF v_dummy > 255 THEN
      RETURN(-1);
    END IF;
    v_part1:=16777216 * v_dummy;
    v_dummy:=SUBSTR(IPADDR,INSTR(IPADDR,'.',1,1)+1,(INSTR(IPADDR,'.',1,2)-1 - INSTR(IPADDR,'.',1,1)));
    IF v_dummy > 255 THEN
      RETURN(-2);
    END IF;
    v_part2:=65536 * v_dummy;    
    v_dummy:=SUBSTR(IPADDR,INSTR(IPADDR,'.',1,2)+1,(INSTR(IPADDR,'.',1,3)-1 - INSTR(IPADDR,'.',1,2)));
    IF v_dummy > 255 THEN
      RETURN(-3);
    END IF;
    v_part3:=256 * v_dummy;
    v_dummy:=SUBSTR(IPADDR,INSTR(IPADDR,'.',1,3)+1);
    IF v_dummy > 255 THEN
      RETURN(-4);
    END IF;
    v_part4:=v_dummy;
    RETURN((v_part1+v_part2+v_part3+v_part4));
  EXCEPTION WHEN OTHERS THEN
    RETURN(0);
  END IP2LONG;

  /*
   *  Function takes an ipaddress as long value and returns the corresponding
   *  country code or NULL in case no match could be found.
   */

  FUNCTION GETCC(IPADDR NUMBER) RETURN VARCHAR2
  IS
    CC  VARCHAR2(2);
  BEGIN
    SELECT COUNTRYCODE INTO CC FROM GEOIP WHERE IPADDR BETWEEN LONGFROM AND LONGTO;
    RETURN(CC);
  EXCEPTION WHEN NO_DATA_FOUND THEN
    RETURN(NULL);
  END GETCC;

  /*
   *  Function takes an ipaddress in dotted format and returns the corresponding
   *  country code or NULL in case no match could be found.
   */

  FUNCTION GETCC(IPADDR VARCHAR2) RETURN VARCHAR2
  IS
  BEGIN
    RETURN(GETCC(IP2LONG(IPADDR)));
  END GETCC;

  /*
   *  Function takes an ipaddress as long value and returns the corresponding
   *  country name or NULL in case no match could be found.
   */
  
  FUNCTION GETCOUNTRYNAME(IPADDR NUMBER) RETURN VARCHAR2
  IS
    CNAME  VARCHAR2(60);
  BEGIN
    SELECT COUNTRYNAME INTO CNAME FROM GEOIP WHERE IPADDR BETWEEN LONGFROM AND LONGTO;
    RETURN(CNAME);
  EXCEPTION WHEN NO_DATA_FOUND THEN
    RETURN(NULL);
  END GETCOUNTRYNAME;

  /*
   *  Function takes an ipaddress in dotted format and returns the corresponding
   *  country name or NULL in case no match could be found.
   */

  FUNCTION GETCOUNTRYNAME(IPADDR VARCHAR2) RETURN VARCHAR2
  IS
  BEGIN
    RETURN(GETCOUNTRYNAME(IP2LONG(IPADDR)));
  END GETCOUNTRYNAME;
  
END UTL_GEOIP;
/
COMMIT
/
